/*
 $License:
    Copyright (C) 2011 InvenSense Corporation, All Rights Reserved.
 $
 */

/******************************************************************************
 *
 * $Id:$
 *
 *****************************************************************************/

/**
 *  @defgroup   Gyro bias supervisor
 *  @brief
 *  
 *  @{
 *      @file   gyroBias.c
 *      @brief  
 */

#include <string.h>
#include "gyroBias.h"
#include "progressiveNoMotion.h"
#include "ml.h"
#include "mlFIFO.h"
#include "mlmath.h"
#include "mlsupervisor.h"
#include "mlMathFunc.h"

#undef MPL_LOG_TAG
#define MPL_LOG_TAG "mpl-gyrobias"

#define GYROBIAS_DEBUG 0

/*
    Data Structures
*/
struct _GB {
    int registeredTrackers;
};

/*
    Globals 
*/
extern tMLSupervisorCB ml_supervisor_cb;

struct _GB gbData;

/*
    Functions
*/
tMLError MLSetGyroBiasReset(void)
{
    memset(&gbData, 0, sizeof(gbData));
    return ML_SUCCESS;
}

void ElectBiases(long *outbias, long *inbias1, long *inbias2, float multiplier)
{
    int i;
    for (i = 0; i < 3; i++) {
        if (ABS(inbias1[i] - inbias2[i]) > (multiplier * mlxData.mlGyroBiasErr)
                && mlxData.mlGyroBiasErr < 5 * 65536) {
            outbias[i] = inbias1[i];
        } else {
            outbias[i] = inbias2[i];
        }
    } 
}

tMLError MLSetGyroBias(void)
{    
    long electedBiases[3];
    long *candidateBiases;

    if (GYROBIAS_DEBUG)
        MPL_LOGI("MLSetGyroBias()\n");

    if (MLGetMotionState() == ML_NO_MOTION) {
        candidateBiases = mlxData.mlNoMotionBias;
    } else {
        if (TempCompIsEnabled() 
                && (gbData.registeredTrackers & GYROBIAS_TRACKER_TEMPCOMP)
                && TempCompHaveOffset()) {
            candidateBiases = mlxData.mlGyroTempBias;
        } else {
            candidateBiases = mlxData.mlNoMotionBias;
        }
    }

#ifdef UMPL
    memcpy(electedBiases, candidateBiases, sizeof(electedBiases));
#else /* progressive no motion is not supported in UMPL */
    if (ProgressiveNoMotionIsEnabled() 
            && (gbData.registeredTrackers & GYROBIAS_TRACKER_PROGRESSIVENOMOTION)) {
        float multiplier = (mlxData.mlGotNoMotionBias == TRUE) ? 1.5f : 1.f;
        ElectBiases(electedBiases, 
            candidateBiases, mlxData.mlProgNoMotionBias, 
            multiplier);
        ProgressiveNoMotionBiasChanged();
    } else {
        memcpy(electedBiases, candidateBiases, sizeof(electedBiases));
    }
#endif
    /* add more comparisons here 
    if (condition?) {
        memcpy(candidateBiases, electedBiases, sizeof(electedBiases));
        ElectBiases(electedBiases, 
            candidateBiases, mlxData.mlGyroTempBias, 
            multiplier);
    }
    */

    MLSetGyroBiasFixed( electedBiases);
    return ML_SUCCESS;
}

/**
 *  @brief  register gyro bias tracker.
 *          Allows tracking of active bias trackers and removal of the 
 *          MLSetGyroBias callback when unnecessary.
 *  @param  add
 *              1 to add or 0 to remove the tracker.
 *  @param  trackerMask
 *              the tag associated with the bias tracker.
 *  @return ML_SUCCESS == 0.
 */
tMLError GyroBiasRegisterTracker(int add, unsigned short trackerMask)
{
    if (add)
        gbData.registeredTrackers |= trackerMask;
    else 
        gbData.registeredTrackers &= ~trackerMask;

    if (gbData.registeredTrackers == 0)
        ml_supervisor_cb.MLGyroBiasSupervisor = NULL;
    else 
        ml_supervisor_cb.MLGyroBiasSupervisor = MLSetGyroBias;

    return ML_SUCCESS;
}

/**
 *  @}
 */

