/*
 $License:
    Copyright (C) 2010 InvenSense Corporation, All Rights Reserved.
 $
 */
/*******************************************************************************
 *
 * $Id: mlos_win32.c 4598 2011-01-25 19:33:13Z prao $
 *
 *******************************************************************************/

/**
 *  @defgroup MLOS
 *  @brief OS Interface for Atmel AVR32
 *
 *  @{
 *      @file mlos.c
 *      @brief OS Interface.
 */

/* ------------- */
/* - Includes. - */
/* ------------- */

#include "mlos.h"
#include "stdint_invensense.h"
#include <asf.h>

/* -------------- */
/* - Functions. - */
/* -------------- */


/**
 *  @brief  Allocate space.
 *  @param  numBytes  
 *              number of bytes.
 *  @return pointer to allocated space.
 */
void *MLOSMalloc(unsigned int numBytes)
{
    MPL_LOGE("MLOSMalloc should not be used");
    return NULL;
}


/**
 *  @brief  Free allocated space.
 *  @param  ptr pointer to space to deallocate
 *  @return error code.
 */
tMLError MLOSFree(void *ptr)
{
    MPL_LOGE("MLOSFree should not be used");
    return ML_SUCCESS;
}





/**
 *  @brief  open file
 *  @param  filename    name of the file to open.
 *  @return error code.
 */
FILE *MLOSFOpen(char *filename)
{
    MPL_LOGE("MLOSFOpen should not be used");
    return NULL;
}


/**
 *  @brief  close the file.
 *  @param  fp  handle to file to close.
 *  @return error code.
 */
void MLOSFClose(FILE *fp)
{
    MPL_LOGE("MLOSFClose should not be used");
    return;
}


/**
 *  @brief  Sleep function.
**/
void MLOSSleep(int mSecs)
{
    delay_ms(mSecs);
}


/**
 *  @brief  get system's internal tick count.
 *          Used for time reference.
 *  @return current tick count.
**/
unsigned long MLOSGetTickCount(void)
{
	const long cpu_hz = 12000000;
	long count, ms;
	count = Get_system_register(AVR32_COUNT);
	ms = cpu_cy_2_ms(count,cpu_hz);
	return ms;
}

/**
 *
 * MUTEX Stubs
 *
**/
tMLError MLOSCreateMutex(HANDLE *mutex) 
{
    return ML_SUCCESS;
}

tMLError MLOSLockMutex(HANDLE mutex)
{
    return ML_SUCCESS;
}

tMLError MLOSUnlockMutex(HANDLE mutex)
{
    return ML_SUCCESS;
}

tMLError MLOSDestroyMutex(HANDLE handle)
{
    return ML_SUCCESS;
}

  /**********************/
 /** @} */ /* defgroup */
/**********************/


