/*
 $License:
    Copyright (C) 2010 InvenSense Corporation, All Rights Reserved.
 $
 */
/*******************************************************************************
 *
 * $Id: aio.h 3863 2010-10-08 22:05:31Z nroyer $
 *
 *******************************************************************************/

#ifndef _AIO_H
#define _AIO_H
#include "mltypes.h"
#ifdef WIN32
#include <windows.h>
#endif
#ifdef LINUX
#include <pthread.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif


    /* ------------ */
    /* - Defines. - */
    /* ------------ */

    /* - Error Codes. - */

#define AIO_SUCCESS 0
#define AIO_ERROR   1

    /* ---------- */
    /* - Enums. - */
    /* ---------- */

    /* --------------- */
    /* - Structures. - */
    /* --------------- */

#ifdef WIN32
#define AIO_RWRSP_SIZE   2048
#endif
#ifdef LINUX
#define AIO_RWRSP_SIZE   512
#endif

#define AIO_RBUFF_MAX    128
#define AIO_RBUFF_SIZE   1024

#define AIO_RBUFF_DYNAMIC  1

#define AIO_INT_MAX        5
#define AIO_INT_SIZE       1

#if defined(WIN32)
    typedef struct {

        char port[255];
        unsigned int baud;

        HANDLE  aioHndl;
        HANDLE  hDevice;                    // handle to the drive to be examined
        OVERLAPPED ovRead;                  // overlap structure for reading I/O
        OVERLAPPED ovWrite;                 // overlap structure for writing I/O
        HANDLE  hReadSemaphore;             // Semaphore signaling data ready
        HANDLE  hReadEvent;                 // event for detecting received data

        int     rbLen;
        int     rbCnt;

        unsigned char readBuffer[2048];
        unsigned char writeBuffer[1024];

        int     rBuffRIndex;
        int     rBuffWIndex;
#if !AIO_RBUFF_DYNAMIC
        unsigned char      rBuff[AIO_RBUFF_MAX][AIO_RBUFF_SIZE];   // Data Stream Buffer
#else
        unsigned char **rBuff;
#endif
        int rBuffMax;
        int rBuffNumBytes;

        unsigned char runThread;

        int       rwRspCnt;
        unsigned char rwRsp[AIO_RWRSP_SIZE];   // Read/Write Response Buffer
        unsigned char rwRspReady;

        unsigned char intSrc;
        unsigned int  intCnt[AIO_INT_MAX];

#define AIO_MODE_DS    0x00000001
#define AIO_MODE_INT   0x00000002
#define AIO_MODE_DM    0x00000004

        unsigned int mode;

        void(*dataStreamHandler)(unsigned char*, int); // StreamHandler callback

    } tAIOVars,     // new type definition
      AIO_Vars_t;   // backward-compatible type definition

#elif defined(LINUX)

    typedef struct {

        char port;
        uint32_t baud;
        
        void*  aioHndl;
        int  hDevice;                    // handle to the drive to be examined 
        pthread_mutex_t read_mutex;
        pthread_cond_t  read_cv;
        
        
        int     rbLen;            
        int     rbCnt;                 
        
        uint8_t      readBuffer[2048];
	uint8_t      writeBuffer[1024];
        
        int     rBuffRIndex;                            
	int     rBuffWIndex;                  
#if !AIO_RBUFF_DYNAMIC 
        uint8_t      rBuff[AIO_RBUFF_MAX][AIO_RBUFF_SIZE];   // Data Stream Buffer
#else
        uint8_t     *rBuff;
#endif
        int       rBuffMax;
	int       rBuffNumBytes;
        
        uint8_t      runThread;
        uint8_t      autoProcess;
        
        int       rwRspCnt;
        uint8_t      rwRsp[AIO_RWRSP_SIZE];                  // Read/Write Response Buffer
        uint8_t      rwRspReady;
        
        uint8_t      intSrc;
        uint32_t     intCnt[AIO_INT_MAX];
        
#define AIO_MODE_DS    0x00000001
#define AIO_MODE_INT   0x00000002
#define AIO_MODE_DM    0x00000004

        uint32_t     mode;
        
        void(*dataStreamHandler)(uint8_t*);                 // StreamHandler callback
    } tAIOVars,     // new type definition
      AIO_Vars_t;   // old type definition
#endif

    /* --------------------- */
    /* - Function p-types. - */
    /* --------------------- */
    int            AIOSetCom         (char const * port, unsigned long dwBaud);
    int            AIOInit           (char const * port, unsigned long dwBaud);
    int            AIOUartRead       (void * lpBuffer, unsigned long nBufferSize, unsigned long * lpBytesReturned);
    int            AIOUartWrite      (void * lpBuffer, unsigned long nBufferSize, unsigned long * lpBytesWritten);
    int            AIOIsOpen         (void);
    int            AIOUartFlush      (void);
    unsigned long  AIOUartGetQStatus (void);

    int AIOOpen           (unsigned char autoProcess, unsigned char createThread);
    int AIOClose          (void);
    int AIODeviceIoControl(void);
    int AIORead           (void);
    int AIOWrite          (void);
    int AIOSetBufferSize  (unsigned short bufferSize);
    int AIOUpdateBuffer   (void);
    int AIOBufferUpdate   (void);
    int AIOHandler        (void);
    int AIOReadBuffer     (unsigned short cnt, unsigned char bufferMode, unsigned char *rBuff);
    int AIOEmptyBuffer    (void);
    int AIOPktsInBuffer   (unsigned short *pktsInBuffer);
    int AIOBytesInBuffer  (int *bytesInBuffer);
    int AIOCreateMutex    (void);
    int AIOLockMutex      (void);
    int AIOUnlockMutex    (void);

    unsigned long AIOUartGetQStatusRwRsp(void);
    tMLError AIOUartReadRwRsp(void * lpBuffer, unsigned long nBufferSize, unsigned long* lpBytesReturned);
    unsigned long AIOUartWaitRwRsp(unsigned long timeout);
#ifdef WIN32
    void AIOSetStreamHandler( void(*func)(unsigned char*, int) );
#endif

    void AIOStopThread    (void);
    void AIORunThread     (void);


#ifdef __cplusplus
}
#endif

#endif // _AIO_H
