#include <asf.h> 
#include <stdio.h>		

#include "mlsl.h"
#include "umpl-tests.h"
#define DEF_PACKET_THRESH 75     // 600 ms / 8ms/sample
#define DEF_TIMING_TOL    0.03f  // 3%
#define DEF_BIAS_THRESH   5242   // 40 dps * 131.072 LSB/dps
#define DEF_RMS_THRESH    687.19f // (.2 dps * 131.072) ^ 2
#define IMU_ADDR        (0x68)
struct {    
    int packet_thresh;         
    float timing_tolerance;    
    int bias_thresh;           
    float RMS_thresh;          
} test_setup = {DEF_PACKET_THRESH,
                DEF_TIMING_TOL,
                DEF_BIAS_THRESH,
                DEF_RMS_THRESH };


int testGyro(void)
{
	int total_count = 0;
	int packet_count;
	unsigned char dataout[2];
	unsigned char gyroData[6];
	int temperature, tmp, i, j;
	int x[260] = {0};
	int y[260] = {0};
	int z[260] = {0};
	long xSum = 0;
	long ySum = 0;

    long zSum = 0;
	float  xAvg, yAvg, zAvg;
	float xRMS, yRMS, zRMS;
	int retVal = 0;
	MPL_LOGD("Initialising test value.\n");
	
	//sensor_bus_put(gyro.hal, 0x15, 0x07);// sample rate = 8ms
	MLSLSerialWriteSingle(0, IMU_ADDR, 0x15, 0x07);
	//sensor_bus_put(gyro.hal, 0x16, 0x03); // full scale = 250dps, filter = 42Hz, analog_sample rate = 1 KHz	
	MLSLSerialWriteSingle(0, IMU_ADDR,0x16, 0x03);
	MLSLSerialRead (0,IMU_ADDR, 0x1b,
					 sizeof (dataout),dataout);		//Read Temperature
	tmp = ((int)dataout[0] * 256 + (int)dataout[1]);
    if (tmp>32767) tmp-=65536;
    temperature += tmp;
	MPL_LOGI("Reading Temperature: %d\n", temperature);
	//First, timing test
	for ( j = 0; j < 3; j++)
	{
		// turn on all gyros, use gyro X for clocking
		//Set to Y and Z for 2nd and 3rd iteration
		//sensor_bus_put(gyro.hal, 0x3e, j+1);
		MLSLSerialWriteSingle(0, IMU_ADDR,0x3e, j+1);
		delay_ms(50);//Wait for 50 ms

		// Enable XYZ gyro in FIFO and nothing else
		MLSLSerialWriteSingle(0, IMU_ADDR, 0x12, 0x70);

		MLSLSerialWriteSingle(0, IMU_ADDR, 0x13, 0);

		// enable FIFO
		MLSLSerialWriteSingle(0, IMU_ADDR, 0x3d, 0x42);
		
		delay_ms(600);//Wait for 600 ms

		// Stop putting data in FIFO
		MLSLSerialWriteSingle(0, IMU_ADDR, 0x12, 0);
		

		MLSLSerialRead(0, IMU_ADDR, 0x3a, sizeof (dataout), dataout );//Getting number of bytes in FIFO

		packet_count = ((int)dataout[0] * 256 + (int)dataout[1]) / 6;
		MPL_LOGI("Packet Count: %d \n", packet_count);
		if (abs(packet_count - test_setup.packet_thresh) <= test_setup.timing_tolerance * test_setup.packet_thresh) //Within +-5% range
		{
			for (  i = 0 ; i < packet_count; i++)
			{
				MLSLSerialRead(0, IMU_ADDR, 0x3c, sizeof (gyroData), gyroData );//Getting FIFO data
				x[total_count+i] = (int)gyroData[0] * 256 + (int)gyroData[1];
				x[total_count+i] = (x[total_count+i] > 32767 ? x[total_count+i] - 65536 : x[total_count+i]);
				y[total_count+i] = (int)gyroData[2] * 256 + (int)gyroData[3];
				y[total_count+i] = (y[total_count+i] > 32767 ? y[total_count+i] - 65536 : y[total_count+i]);
				z[total_count+i] = (int)gyroData[4] * 256 + (int)gyroData[5];
				z[total_count+i] = (z[total_count+i] > 32767 ? z[total_count+i] - 65536 : z[total_count+i]);
			}
			total_count += packet_count;
		}
		else
		{
			// Exit timing failure
			//return j+1;
			retVal |= 1 << j;
			MPL_LOGI("Incorrect packet count \n");			
		}
		MLSLSerialRead (0, IMU_ADDR, 0x1b,
        sizeof (dataout), dataout );		//Read Temperature
		tmp = ((int)dataout[0] * 256 + (int)dataout[1]);
		if (tmp>32767) tmp-=65536;
		temperature += tmp;		  
		MPL_LOGI("Reading Temperature: %d\n", temperature);
	}
	//2nd, check for bias
	for ( i = 0; i < total_count ; i++)
	{
		xSum += x[i]; ySum += y[i]; zSum += z[i];
	}
	MPL_LOGI("xSum: %d,  ySum: %d,  zSum: %d \n", xSum, ySum, zSum); 
	if (total_count != 0)
	{
		xAvg = ((float)xSum / total_count); yAvg = ((float)ySum / total_count); zAvg = ((float)zSum / total_count);
	}
	else
	{
		xAvg = (float)xSum;
		yAvg = (float)ySum;
		zAvg = (float)zSum;
	}
	MPL_LOGI("xAvg: %f,  yAvg: %f,  zAvg: %f \n", xAvg, yAvg, zAvg); 
	if ( xAvg > test_setup.bias_thresh
		|| xAvg < -1 * test_setup.bias_thresh)
	{
		MPL_LOGI("X-Gyro bias exceeded threshold (threshold = %d)\n", test_setup.bias_thresh);
		retVal |= 1 << 3;
	}
	if ( yAvg > test_setup.bias_thresh
		|| yAvg < -1 * test_setup.bias_thresh)
	{
		MPL_LOGI("Y-Gyro bias exceeded threshold (threshold = %d)\n", test_setup.bias_thresh);
		retVal |= 1 << 4;
	}
	if ( zAvg > test_setup.bias_thresh
		|| zAvg < -1 * test_setup.bias_thresh)
	{
		MPL_LOGI("Z-Gyro bias exceeded threshold (threshold = %d)\n", test_setup.bias_thresh);
		retVal |= 1 << 5;
	}

	xRMS = 0; yRMS = 0; zRMS = 0;

	//Finally, check RMS
	for ( i = 0; i < total_count ; i++)
	{
		xRMS += (x[i]-xAvg)*(x[i]-xAvg);
		yRMS += (y[i]-yAvg)*(y[i]-yAvg);
		zRMS += (z[i]-zAvg)*(z[i]-zAvg);        
	}
	
	if ( xRMS > test_setup.RMS_thresh * total_count)
	{
		MPL_LOGI("X-Gyro RMS exceeded threshold (threshold = %f)\n", test_setup.RMS_thresh * total_count);
		retVal |= 1 << 6;
	}
	if ( yRMS > test_setup.RMS_thresh * total_count )
	{	
		MPL_LOGI("Y-Gyro RMS exceeded threshold (threshold = %f)\n", test_setup.RMS_thresh * total_count);
		retVal |= 1 << 7;
	}
	if ( zRMS > test_setup.RMS_thresh * total_count )
	{
		MPL_LOGI("Z-Gyro RMS exceeded threshold (threshold = %f)\n", test_setup.RMS_thresh * total_count);
		retVal |= 1 << 8;
	}

    //In order to preserve precision, we don't do square-root and all the division here.
    MPL_LOGI("XRMS Test: %f,  YRMS Test: %f,  ZRMS Test: %f \n", xRMS, yRMS, zRMS); 
	if ( xRMS == 0 || yRMS == 0 || zRMS == 0)
		//If any of the RMS noise value returns zero, then we might have dead gyro or FIFO/register failure
		retVal |= 1 << 9;

    temperature /= 4;        
    MPL_LOGI("Temperature: %f\n", 35.0f+((((float)temperature)+13200.0f)/280.0f));
	return retVal; //Return success
	
}


