
#include <stdio.h>
#include <string.h>

#include "umpl-setup.h"

#include "ml.h"
#include "mldl.h"
#include "mlstates.h"
#include "mlFIFO.h"
#include "mpu.h"
#include "mldl_cfg.h"
#include "slave.h"
#include "tempComp.h"

#include "log.h"
#undef MPL_LOG_TAG
#define MPL_LOG_TAG "umpl-setup"

/* found in mldl.c */
extern struct ext_slave_descr gAccel;
extern struct mpu_platform_data gPdata;

/* SetupPlatform
 * To be run before MLDmpOpen()
 */

tMLError SetupPlatform(void)
{
    struct ext_slave_descr* accel = NULL;
    tMLError result;

    memset(&gPdata.orientation,0,sizeof(gPdata.orientation));
    gPdata.orientation[0] = 1;
    gPdata.orientation[4] = -1;
    gPdata.orientation[8] = -1;
    gPdata.int_config = 0x10;  // FIXME - rolldice default, not sure if applicable

    memset(&gAccel,0,sizeof(gAccel));

    accel = kxtf9_get_slave_descr();
    RANGE_FLOAT_TO_FIXEDPOINT(accel->range,2.f);

    memset(gPdata.accel.orientation,0, sizeof(gPdata.accel.orientation));
    gPdata.accel.orientation[1] = 1;
    gPdata.accel.orientation[3] = 1;
    gPdata.accel.orientation[8] = -1;

    gPdata.accel.adapt_num = 0;
    gPdata.accel.bus = EXT_SLAVE_BUS_SECONDARY;
    gPdata.accel.get_slave_descr = kxtf9_get_slave_descr;
    gPdata.accel.address = ACCEL_SLAVEADDR_KXTF9;
    
    memcpy(&gAccel, accel, sizeof(*accel));

    return ML_SUCCESS;
}

/* SetupDmpOpen
 * To be run after MLDmpOpen(), before MLDmpStart().
 */

tMLError SetupDmpOpen(void)
{
    tMLError result;
 
    // Setup Bias Trackers: 
    /* MLSetBiasUpdateFunc
     * Known to work with the following arguments:
     * ML_NONE: V/MPL-ml forces disable of PROGRESSIVE_NO_MOTION for some reason.
     * ML_BIAS_FROM_NO_MOTION: same err as above.
     *
     */

    result = MLSetBiasUpdateFunc( ML_BIAS_FROM_NO_MOTION
                                | ML_BIAS_FROM_TEMPERATURE
                                | ML_LEARN_BIAS_FROM_TEMPERATURE ); 
    if (result != ML_SUCCESS)
        MPL_LOGE("umpl MLSetBiasUpdateFunc returned %d\n",result);
      
    // Setup FIFO:
    result = FIFOSendQuaternion(ML_32_BIT);
    if (result != ML_SUCCESS)
        MPL_LOGE("umpl FIFOSendQuaterion returned %d\n",result);
    
    result = FIFOSendGyro(ML_ALL,ML_32_BIT);
    if (result != ML_SUCCESS)
        MPL_LOGE("umpl FIFOSendGyro returned %d\n",result);
    
    result = FIFOSendAccel(ML_ALL,ML_32_BIT);
    if (result != ML_SUCCESS)
        MPL_LOGE("umpl FIFOSendAccel returned %d\n",result);

    result = MLSetFIFORate(10);
    if (result != ML_SUCCESS)
        MPL_LOGE("umpl MLSetFIFORate returned %d\n",result);

    result = MLEnableTempComp();
    if (result != ML_SUCCESS)
       MPL_LOGE("umpl MLEnableTempComp returned %d\n",result);
    return ML_SUCCESS;
}

