
#ifndef __UMPL_STATES_H__
#define __UMPL_STATES_H__

#include "mltypes.h"

/* Three power states:
 *  3.  UMPL_RUN      : MPU + Accel sensor fusion
 *  2.  UMPL_LP_ACCEL : Accel only with motion interrupt 
 *  1.  UMPL_STOP     : All sensors off (suitable for host processor asleep)
 *  0.  UMPL_UNINIT   : Unititialized (only on boot)
 *
 * Transitions:
 *  3 -> 2 : ML_NO_MOTION timeout   or Call to umplStopMPU
 *  2 -> 3 : Accel interrupt occurs or Call to umplStartMPU
 *  2 -> 1 : Call to umplStop
 *  3 -> 1 : Call to umplStop
 *  1 -> 2 : Call to umplStartAccel
 *  1 -> 3 : Call to umplStartMPU
 *  0 -> 1 : Call to umplInit
 *  (0 is the state on boot)
 */

#define UMPL_UNINIT   (0)
#define UMPL_STOP     (1)
#define UMPL_LP_ACCEL (2)
#define UMPL_RUN      (3)

int umplGetState(void);

tMLError umplInit(unsigned char *port);
tMLError umplStartMPU(void);
tMLError umplStartAccel(void);
tMLError umplStopMPU(void);
tMLError umplStop(void);

tMLError umplNotifyInterrupt(unsigned char interrupt);
tMLError umplOnTimer(void);

#endif

